/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifdef __GNUC__
#pragma implementation
#endif

#include "Window.h"
#include <new.h>

#include <Xm/RowColumn.h>
#include <Xm/DialogS.h>
#include <Xm/MainW.h>
#include <Xm/CascadeB.h>
#include <Xm/SeparatoG.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/Form.h>
#include <Xm/ScrolledW.h>

void xwWindow::cbDestroy(Widget w, XtPointer client_data, XtPointer call_data)
   {xwWindow* me = (xwWindow*)client_data;
    // if (me->handler) me->handler->onDestroy(me, me->handler_arg);
    delete me;
   }

xwWindow::xwWindow(xwApp* app, const char* name, const xwMenu* menu)
   {menuitem=(Menuitem*)0;
    parent_app = app;
/*
    shell = XtVaAppCreateShell("window", "shell"
		, topLevelShellWidgetClass, app->getDisplay()
		, XmNtitle, name
		, 0);

    widget = XtVaCreateManagedWidget(name
		, xmMainWindowWidgetClass, shell
		, 0);
*/
    shell = XtVaCreatePopupShell("window"
		, topLevelShellWidgetClass,	app->getWindow()
		, XmNtitle,			name
		, NULL);

    widget = XtVaCreateWidget(name
		, xmMainWindowWidgetClass, shell
		, NULL);

    XtAddCallback(widget, XmNdestroyCallback, cbDestroy, (XtPointer)this);

    default_callbacks(0, meth_Caption|meth_Size);
    ((xwCore*)app)->addWidget(this);

    if (menu)
       {Widget menubar = XmCreateMenuBar(widget, "MenuBar", NULL, 0);
	addMenu(menubar, menu);
	XtManageChild(menubar);
   }   }

void xwWindow::addMenu(Widget parent, const xwMenu*& menu)
   {Widget    w;
    XmString ls;
    Menuitem*mi;
    while (menu->type!=xwMenu::END) switch (menu->type)
       {case xwMenu::item:
	    mi = new Menuitem();
	    mi->me    = this;
	    mi->index = menu->index;
	    mi->w = w = XtVaCreateManagedWidget(menu->name
		, xmPushButtonGadgetClass, parent
		, XmNmnemonic, menu->shortcut
		, NULL);
	    mi->next  = menuitem; menuitem=mi;
	    XtAddCallback(w, XmNactivateCallback, cbMenu, (XtPointer)mi);
	    menu++;
	    break;
	case xwMenu::submenu:
	    w = XmCreatePulldownMenu(parent, "Pulldown", NULL, 0);
	    ls = XmStringCreateLocalized((char*)menu->name);
	    XtVaCreateManagedWidget(menu->name
		, xmCascadeButtonWidgetClass, parent
		, XmNlabelString, ls
		, XmNmnemonic, menu->shortcut
		, XmNsubMenuId, w
		, (void*)0);
	    XmStringFree(ls);
	    menu++;
	    addMenu(w, menu);
	    break;
	case xwMenu::option:
	    mi = new Menuitem();
	    mi->me    = this;
	    mi->index = menu->index;
	    mi->w = w = XtVaCreateManagedWidget(menu->name
		, xmToggleButtonGadgetClass, parent
		, XmNmnemonic, menu->shortcut
		, NULL);
	    mi->next  = menuitem; menuitem=mi;
	    XtAddCallback(w, XmNvalueChangedCallback, cbMenu, (XtPointer)mi);
	    menu++;
	    break;
	case xwMenu::seperator:
	    XtVaCreateManagedWidget("separator"
		, xmSeparatorGadgetClass, parent, NULL);
	    menu++;
	    break;
	default:
	    menu++;
       }
    menu++;
   }


xwWindow::~xwWindow()
   {
    if (handler) handler->onDestroy(this, handler_arg);
    if (widget)
       {XtRemoveCallback(widget, XmNdestroyCallback, cbDestroy, (XtPointer)this);
	XtDestroyWidget(widget);
       }
    XtDestroyWidget(shell);
    ((xwCore*)parent_app)->removeWidget(this);
    destroyWidgets();
    while (menuitem)
	  {Menuitem* ptr = menuitem;
	   menuitem = ptr->next;
	   delete ptr;
   }	  }

void xwWindow::show()
   {XtManageChild(widget);
    XtManageChild(shell);
    XtRealizeWidget(shell);
   }

void xwWindow::addCommand(xwWidget* w)
   {XtVaSetValues(widget
		, XmNcommandWindow, w->getWidget()
		, 0);
   }

void xwWindow::cbMenu(Widget w, XtPointer client_data, XtPointer call_data)
   {Menuitem* mi = (Menuitem*)client_data;
    xwWindow* me = mi->me;
    if (me->handler) me->handler->onMenu(me, mi->index);
   }

void xwWindow::menuSetOption(int id, int value)
   {Menuitem* mi;
    for (mi=menuitem; mi && mi->index!=id; ) mi=mi->next;
    if (mi) XtVaSetValues(mi->w, XmNset, (unsigned char)value, 0);
   }
int xwWindow::menuGetOption(int id)
   {Menuitem* mi; unsigned char set=0;
    for (mi=menuitem; mi && mi->index!=id; ) mi=mi->next;
    if (mi) XtVaSetValues(mi->w, XmNset, &set, 0);
    return set;
   }
void xwWindow::menuSetEnable(int id, int value)
   {Menuitem* mi;
    for (mi=menuitem; mi && mi->index!=id; ) mi=mi->next;
    if (mi) XtVaSetValues(mi->w, XmNsensitive, (Boolean)value, 0);
   }

void xwWindow::move(int x, int y)
   {XtVaSetValues(shell, XmNx, (Position)x, XmNy, (Position)y, NULL);
   }

#include <iostream.h>

void xwWindow::center()
   {xwApp* app = getApp();
    int ww, wh;
    int sw, sh;
    getSize(&ww, &wh);
    sw = app->screenWidth();
    sh = app->screenHeight();

    if (ww>sw || wh>sh)
	 move(0, 0);
    else move((sw-ww)/2, (sh-wh)/2);
   }

